/*
 *
 * Copyright 1994 Sun Microsystems, Inc. All Rights Reserved
 * ident "@(#)receiver.c 1.2 94/04/06 Copyright 06 Apr 1994 SMI"
 *
 * 
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#define PORT 		5550
#define DEF_TCP_BUF	32	/* Kytes: TCP buffer size */
extern char *optarg;
extern int optind, opterr, optopt;



void usage ();

/* ------------------------ */
void
tvsub(out,in)
struct timeval *out	;
struct timeval *in	;
{
        /*
         * out is the return pointer .
         */
        if ( (out->tv_usec  -= in->tv_usec) < 0) {  /* Substract microsec */
                out->tv_sec-- ;
                out->tv_usec  += 1000000 ;
        }
        out->tv_sec  -=  in->tv_sec   ;  /* Substract seconds */
}
/* ------------------------ */

main(argc,argv)
int argc ;
char **argv ;
{
	struct hostent 		*h;
	struct sockaddr_in 	server;
	int 			sock;
	char 			*buffer;
	long 			tcp_buf_size = DEF_TCP_BUF * 1024;
	int 			nbytes = 0		;
	struct timeval 		tv1 , tv2;
	int 			start=1;
	int 			c;
	char 			*host;
	long 			count = 0	;

	/* 
	 * Usage
	 */
	 while ((c = getopt(argc, argv, "b:")) != EOF)
               switch (c) {
               case 'b':
                    tcp_buf_size =atoi(optarg) * 1024 ;
                    break;
               default: 
		    usage(argv[0]);
		    break;
          }
	if ( argc - optind != 1) {
		usage(argv[0]);
	}
	host=argv[optind];
	
	bzero((char *)&tv1,sizeof(struct timeval));
	bzero((char *)&tv2,sizeof(struct timeval));
	bzero((char *)&server, sizeof(struct sockaddr_in));

	server.sin_addr.s_addr = inet_addr(host);

	if (server.sin_addr.s_addr == -1 || server.sin_addr.s_addr == 0) { 
		h = gethostbyname(host);
		if (h == NULL) {
			(void)fprintf(stderr,"gethostbyname fails\n");
			exit(2);
		}
		bcopy(h->h_addr,(caddr_t)&server.sin_addr,h->h_length) ;   

	}

	server.sin_family  =  AF_INET ;
	server.sin_port    = htons(PORT);

	sock = socket(AF_INET, SOCK_STREAM,0);
	if (sock == -1) {
		perror("	***socket");
		exit(3);
		}

	(void)printf("TEST: Receiving data from remote host <%s> \n",host);
        (void)printf("      TCP receiving buffer: %5d bytes\n",tcp_buf_size);
         
	/*
         * Allocate the buffer
	 */
	buffer = (char *) calloc (tcp_buf_size,sizeof(char));
        if (buffer == NULL) {
                perror("        ***calloc");
                exit(1); 
                }
         
	/*
	 * Set the TCP buffer
	 */
	if (setsockopt(sock,SOL_SOCKET,SO_RCVBUF,
			(char *)&tcp_buf_size,sizeof(int)) < 0) {
			perror("	***setsockopt");
			exit(4);
			}

	/*
	 * Connect
	 */
	if (connect(sock,(struct sockaddr *)&server,sizeof (struct sockaddr_in)) < 0) {
		perror("	***connect");
		exit(5);
		}
	(void)printf("connect OK \n");


	/*
	 * Loop: receive data 
	 */
	gettimeofday(&tv1,(struct timezone *)0) ;

	while ( (nbytes = recv(sock,buffer,tcp_buf_size,0)) > 0 ) {
		if ( start ) { 
			start = 0 ;
			(void)printf("Start receiving data ... \n");
		}  
		count += nbytes;

	
		/*      look_at(buffer,nbytes); 
			bzero((char *)buffer,USBUFSIZE);
		*/
		}

	gettimeofday(&tv2,(struct timezone *)0) ;
	tvsub(&tv2,&tv1) ;

	/*
	 * Print Results
	 */
	(void)printf("Data transfer finished\n");
	(void)printf("Transfer duration:  %9d sec \n",tv2.tv_sec);
	(void)printf("Transfer duration:  %9d micro-sec \n",tv2.tv_usec);
	(void)printf("Data transferred:    %9d bytes\n",count);
	if ( tv2.tv_sec > 0 ) { 
		double    msec =  (double) 1000*tv2.tv_sec+tv2.tv_usec/1000;
		double    bits = (double) count * 8 ;
		(void)printf("Data transfer rate:  %G Mbits/sec\n", 
			(double) bits  * 1000 / (double) (msec * 1024 * 1024 ));
	}
	else
		(void)printf("Data transfer rate: transfer too fast - try with more data \n");

	return(0); 
}

void
usage ( name ) 
char * name; 
{
	(void)fprintf(stderr,"Usage: %s [-b <BUF SIZE> ] hostname \n",name);
	(void)fprintf(stderr,"\t-b <Kbytes>: set the size of the TCP buffer used to receive data \n");
	(void)fprintf(stderr,"\thostname:    host where the sender is running \n");
	exit (1);
}


